package api

import (
	"ifms/internal/mods/autocontrol/biz"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 设备管理
type Device struct {
	DeviceBIZ *biz.Device
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Query device list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.Device}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices [get]
func (a *Device) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.DeviceQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DeviceBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Get device record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.Device}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices/{id} [get]
func (a *Device) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.DeviceBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Create device record
// @Param body body schema.DeviceForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.Device}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices [post]
func (a *Device) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	result, err := a.DeviceBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Update device record by ID
// @Param id path string true "unique id"
// @Param body body schema.DeviceForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices/{id} [put]
func (a *Device) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.DeviceBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Delete device record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices/{id} [delete]
func (a *Device) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.DeviceBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Get device statistics
// @Success 200 {object} util.ResponseResult{data=[]schema.DeviceStatistics}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices/statistics [get]
func (a *Device) Statistics(c *gin.Context) {
	ctx := c.Request.Context()
	statistics, err := a.DeviceBIZ.Statistics(ctx)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, statistics)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Device aeration config
// @Param body body schema.DeviceConfigAeration true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceConfigAeration}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices [post]
func (a *Device) AerationConfig(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceConfigAeration)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.DeviceBIZ.AerationConfig(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, nil)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Device feeding config
// @Param body body schema.DeviceConfigFeeding true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceConfigFeeding}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices [post]
func (a *Device) FeedingConfig(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceConfigFeeding)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.DeviceBIZ.FeedingConfig(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, nil)
}

// @Tags DeviceAPI
// @Security ApiKeyAuth
// @Summary Device water circulation config
// @Param body body schema.DeviceConfigWaterCirculation true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceConfigWaterCirculation}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices [post]
func (a *Device) WaterCirculationConfig(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceConfigWaterCirculation)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.DeviceBIZ.WaterCirculationConfig(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, nil)
}
