package api

import (
	"ifms/internal/mods/autocontrol/biz"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 设备-鱼池关联表
type DeviceFishPond struct {
	DeviceFishPondBIZ *biz.DeviceFishPond
}

// @Tags DeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Query device fish pond list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.DeviceFishPond}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-fish-ponds [get]
func (a *DeviceFishPond) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.DeviceFishPondQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DeviceFishPondBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags DeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Get device fish pond record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceFishPond}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-fish-ponds/{id} [get]
func (a *DeviceFishPond) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.DeviceFishPondBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags DeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Create device fish pond record
// @Param body body schema.DeviceFishPondForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceFishPond}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-fish-ponds [post]
func (a *DeviceFishPond) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceFishPondForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	result, err := a.DeviceFishPondBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags DeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Update device fish pond record by ID
// @Param id path string true "unique id"
// @Param body body schema.DeviceFishPondForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-fish-ponds/{id} [put]
func (a *DeviceFishPond) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceFishPondForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.DeviceFishPondBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags DeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Delete device fish pond record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-fish-ponds/{id} [delete]
func (a *DeviceFishPond) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.DeviceFishPondBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
