package api

import (
	"ifms/internal/mods/autocontrol/biz"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 设备类型信息
type DeviceTypeInfo struct {
	DeviceTypeInfoBIZ *biz.DeviceTypeInfo
}

// @Tags DeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Query device type info list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.DeviceTypeInfo}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-type-infos [get]
func (a *DeviceTypeInfo) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.DeviceTypeInfoQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DeviceTypeInfoBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags DeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Get device type info record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceTypeInfo}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-type-infos/{id} [get]
func (a *DeviceTypeInfo) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.DeviceTypeInfoBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags DeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Create device type info record
// @Param body body schema.DeviceTypeInfoForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.DeviceTypeInfo}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-type-infos [post]
func (a *DeviceTypeInfo) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceTypeInfoForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DeviceTypeInfoBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags DeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Update device type info record by ID
// @Param id path string true "unique id"
// @Param body body schema.DeviceTypeInfoForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-type-infos/{id} [put]
func (a *DeviceTypeInfo) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DeviceTypeInfoForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.DeviceTypeInfoBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags DeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Delete device type info record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/device-type-infos/{id} [delete]
func (a *DeviceTypeInfo) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.DeviceTypeInfoBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
