package biz

import (
	"context"
	"strconv"
	"strings"
	"time"

	"ifms/internal/consts"
	"ifms/internal/mods/autocontrol/dal"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"
)

// 设备管理
type Device struct {
	Trans             *util.Trans
	DeviceDAL         *dal.Device
	DeviceFishPondDAL *dal.DeviceFishPond
}

// Query devices from the data access object based on the provided parameters and options.
func (a *Device) Query(ctx context.Context, params schema.DeviceQueryParam) (*schema.DeviceQueryResult, error) {
	params.Pagination = true
	result, err := a.DeviceDAL.Query(ctx, params, schema.DeviceQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	// 预加载鱼池

	return result, nil
}

// Get the specified device from the data access object.
func (a *Device) Get(ctx context.Context, id string) (*schema.Device, error) {
	device, err := a.DeviceDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if device == nil {
		return nil, errors.NotFound("", "Device not found")
	}
	return device, nil
}

// Create a new device in the data access object.
func (a *Device) Create(ctx context.Context, formItem *schema.DeviceForm) (*schema.Device, error) {
	device := &schema.Device{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(device); err != nil {
		return nil, err
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	// 判断设备编号是否存在
	exists, err := a.DeviceDAL.GetByDeviceNo(ctx, device.DeviceNo)
	if err != nil {
		return nil, err
	} else if exists != nil {
		return nil, errors.BadRequest("", "设备编号已存在")
	}
	err = a.Trans.Exec(ctx, func(ctx context.Context) error {

		if err := a.DeviceDAL.Create(ctx, device); err != nil {
			return err
		}
		if len(formItem.DeviceFishPonds) > 0 {
			for _, fishPond := range formItem.DeviceFishPonds {
				deviceFishPond := &schema.DeviceFishPond{
					DeviceID:     device.ID,
					FishPondID:   fishPond.FishPondID,
					FishPondName: fishPond.FishPondName,
					TenantID:     device.TenantID,
				}
				if err := a.DeviceFishPondDAL.Create(ctx, deviceFishPond); err != nil {
					return err
				}
			}
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return device, nil
}

// Update the specified device in the data access object.
func (a *Device) Update(ctx context.Context, id string, formItem *schema.DeviceForm) error {
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	device, err := a.DeviceDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if device == nil {
		return errors.NotFound("", "Device not found")
	}

	if err := formItem.FillTo(device); err != nil {
		return err
	}

	// 判断设备编号是否存在
	exists, err := a.DeviceDAL.GetByDeviceNo(ctx, device.DeviceNo)
	if err != nil {
		return err
	} else if exists != nil && exists.ID != device.ID {
		return errors.BadRequest("", "设备编号已存在")
	}

	device.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceDAL.Update(ctx, device); err != nil {
			return err
		}
		if err := a.DeviceFishPondDAL.DeleteByDeviceID(ctx, device.ID); err != nil {
			return err
		}
		if len(formItem.DeviceFishPonds) > 0 {
			for _, fishPond := range formItem.DeviceFishPonds {
				deviceFishPond := &schema.DeviceFishPond{
					DeviceID:     device.ID,
					FishPondID:   fishPond.FishPondID,
					FishPondName: fishPond.FishPondName,
					TenantID:     device.TenantID,
				}
				if err := a.DeviceFishPondDAL.Create(ctx, deviceFishPond); err != nil {
					return err
				}
			}
		}
		return nil
	})
}

// Delete the specified device from the data access object.
func (a *Device) Delete(ctx context.Context, id string) error {
	exists, err := a.DeviceDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Device not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}

func (a *Device) Statistics(ctx context.Context) ([]schema.DeviceStatistics, error) {
	statistics, err := a.DeviceDAL.Statistics(ctx)
	if err != nil {
		return nil, err
	}
	return statistics, nil
}

func (a *Device) AerationConfig(ctx context.Context, item *schema.DeviceConfigAeration) error {
	// 查询设备
	device, err := a.DeviceDAL.Get(ctx, strconv.FormatInt(item.DeviceID, 10))
	if err != nil {
		return err
	}
	if device == nil {
		return errors.NotFound("", "设备不存在")
	}

	if device.Type != schema.DeviceTypeAeration {
		return errors.BadRequest("", "设备类型错误")
	}
	// 更新设备状态
	device.Status = item.Status
	if err := a.DeviceDAL.Update(ctx, device); err != nil {
		return err
	}

	deviceAeration := &schema.DeviceConfigAeration{
		DeviceID: item.DeviceID,
		TenantID: device.TenantID,
		Level:    item.Level,
		Aeration: item.Aeration,
		Status:   item.Status,
	}
	operation, err := a.DeviceDAL.SaveAerationConfig(ctx, deviceAeration)
	if err != nil {
		return err
	}
	// 记录操作日志
	RelatedPondIds := strconv.FormatInt(device.WorkshopID, 10)
	RelatedPond := device.WorkshopName
	for _, fishPond := range device.DeviceFishPonds {
		RelatedPondIds += strconv.FormatInt(fishPond.FishPondID, 10) + ","
		RelatedPond += fishPond.FishPondName + ","
	}
	// 移除逗号
	RelatedPondIds = strings.TrimSuffix(RelatedPondIds, ",")
	RelatedPond = strings.TrimSuffix(RelatedPond, ",")
	user := util.FromLoginUser(ctx)
	deviceControlLog := &schema.DeviceControlLog{
		TenantID:       user.TenantID,
		DeviceID:       item.DeviceID,
		DeviceModel:    device.Model,
		DeviceName:     device.Name,
		RelatedPondIds: RelatedPondIds,
		RelatedPond:    RelatedPond,
		OperatorUserID: int(user.ID),
		Operator:       user.Username,
		Command:        operation,
		CommandStatus:  true,
		Type:           device.Type,
	}

	deviceControlLogDAL := dal.DeviceControlLog{
		DB: a.DeviceDAL.DB,
	}

	if err := deviceControlLogDAL.Create(ctx, deviceControlLog); err != nil {
		return err
	}
	return nil
}

func (a *Device) FeedingConfig(ctx context.Context, item *schema.DeviceConfigFeeding) error {
	// 查询设备
	device, err := a.DeviceDAL.Get(ctx, strconv.FormatInt(item.DeviceID, 10))
	if err != nil {
		return err
	}
	if device == nil {
		return errors.NotFound("", "设备不存在")
	}
	if device.Type != schema.DeviceTypeFeeding {
		return errors.BadRequest("", "设备类型错误")
	}

	// 更新设备状态
	device.Status = item.Status
	if err := a.DeviceDAL.Update(ctx, device); err != nil {
		return err
	}

	deviceFeeding := &schema.DeviceConfigFeeding{
		TenantID:     device.TenantID,
		DeviceID:     item.DeviceID,
		FeedLevel:    item.FeedLevel,
		FeedTime:     item.FeedTime,
		IntervalTime: item.IntervalTime,
		Status:       item.Status,
	}
	operation, err := a.DeviceDAL.SaveFeedingConfig(ctx, deviceFeeding)
	if err != nil {
		return err
	}
	// 记录操作日志
	RelatedPondIds := strconv.FormatInt(device.WorkshopID, 10)
	RelatedPond := device.WorkshopName
	for _, fishPond := range device.DeviceFishPonds {
		RelatedPondIds += strconv.FormatInt(fishPond.FishPondID, 10) + ","
		RelatedPond += fishPond.FishPondName + ","
	}
	// 移除逗号
	RelatedPondIds = strings.TrimSuffix(RelatedPondIds, ",")
	RelatedPond = strings.TrimSuffix(RelatedPond, ",")
	user := util.FromLoginUser(ctx)
	deviceControlLog := &schema.DeviceControlLog{
		TenantID:       user.TenantID,
		DeviceID:       item.DeviceID,
		DeviceModel:    device.Model,
		DeviceName:     device.Name,
		RelatedPondIds: RelatedPondIds,
		RelatedPond:    RelatedPond,
		OperatorUserID: int(user.ID),
		Operator:       user.Username,
		Command:        operation,
		CommandStatus:  true,
		Type:           device.Type,
	}

	deviceControlLogDAL := dal.DeviceControlLog{
		DB: a.DeviceDAL.DB,
	}

	if err := deviceControlLogDAL.Create(ctx, deviceControlLog); err != nil {
		return err
	}
	return nil
}

func (a *Device) WaterCirculationConfig(ctx context.Context, item *schema.DeviceConfigWaterCirculation) error {
	// 查询设备
	device, err := a.DeviceDAL.Get(ctx, strconv.FormatInt(item.DeviceID, 10))
	if err != nil {
		return err
	}
	if device == nil {
		return errors.NotFound("", "设备不存在")
	}
	if device.Type != schema.DeviceTypeWaterCirculation {
		return errors.BadRequest("", "设备类型错误")
	}
	// 更新设备状态
	device.Status = item.Status
	if err := a.DeviceDAL.Update(ctx, device); err != nil {
		return err
	}

	deviceWaterCirculation := &schema.DeviceConfigWaterCirculation{
		TenantID:         device.TenantID,
		DeviceID:         item.DeviceID,
		WaterTemp:        item.WaterTemp,
		WaterCirculation: item.WaterCirculation,
		Aeration:         item.Aeration,
		Filter:           item.Filter,
		Disinfection:     item.Disinfection,
		Status:           item.Status,
	}
	operation, err := a.DeviceDAL.SaveWaterCirculationConfig(ctx, deviceWaterCirculation)
	if err != nil {
		return err
	}
	// 记录操作日志
	RelatedPondIds := strconv.FormatInt(device.WorkshopID, 10)
	RelatedPond := device.WorkshopName
	for _, fishPond := range device.DeviceFishPonds {
		RelatedPondIds += strconv.FormatInt(fishPond.FishPondID, 10) + ","
		RelatedPond += fishPond.FishPondName + ","
	}
	// 移除逗号
	RelatedPondIds = strings.TrimSuffix(RelatedPondIds, ",")
	RelatedPond = strings.TrimSuffix(RelatedPond, ",")
	user := util.FromLoginUser(ctx)
	deviceControlLog := &schema.DeviceControlLog{
		TenantID:       user.TenantID,
		DeviceID:       item.DeviceID,
		DeviceModel:    device.Model,
		DeviceName:     device.Name,
		RelatedPondIds: RelatedPondIds,
		RelatedPond:    RelatedPond,
		OperatorUserID: int(user.ID),
		Operator:       user.Username,
		Command:        operation,
		CommandStatus:  true,
		Type:           device.Type,
	}

	deviceControlLogDAL := dal.DeviceControlLog{
		DB: a.DeviceDAL.DB,
	}

	if err := deviceControlLogDAL.Create(ctx, deviceControlLog); err != nil {
		return err
	}
	return nil
}
