package biz

import (
	"context"
	"time"

	"ifms/internal/mods/autocontrol/dal"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 设备-鱼池关联表
type DeviceFishPond struct {
	Trans             *util.Trans
	DeviceFishPondDAL *dal.DeviceFishPond
}

// Query device fish ponds from the data access object based on the provided parameters and options.
func (a *DeviceFishPond) Query(ctx context.Context, params schema.DeviceFishPondQueryParam) (*schema.DeviceFishPondQueryResult, error) {
	params.Pagination = true

	result, err := a.DeviceFishPondDAL.Query(ctx, params, schema.DeviceFishPondQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified device fish pond from the data access object.
func (a *DeviceFishPond) Get(ctx context.Context, id string) (*schema.DeviceFishPond, error) {
	deviceFishPond, err := a.DeviceFishPondDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if deviceFishPond == nil {
		return nil, errors.NotFound("", "Device fish pond not found")
	}
	return deviceFishPond, nil
}

// Create a new device fish pond in the data access object.
func (a *DeviceFishPond) Create(ctx context.Context, formItem *schema.DeviceFishPondForm) (*schema.DeviceFishPond, error) {
	deviceFishPond := &schema.DeviceFishPond{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(deviceFishPond); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceFishPondDAL.Create(ctx, deviceFishPond); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return deviceFishPond, nil
}

// Update the specified device fish pond in the data access object.
func (a *DeviceFishPond) Update(ctx context.Context, id string, formItem *schema.DeviceFishPondForm) error {
	deviceFishPond, err := a.DeviceFishPondDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if deviceFishPond == nil {
		return errors.NotFound("", "Device fish pond not found")
	}

	if err := formItem.FillTo(deviceFishPond); err != nil {
		return err
	}
	deviceFishPond.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceFishPondDAL.Update(ctx, deviceFishPond); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified device fish pond from the data access object.
func (a *DeviceFishPond) Delete(ctx context.Context, id string) error {
	exists, err := a.DeviceFishPondDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Device fish pond not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceFishPondDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
