package biz

import (
	"context"
	"time"

	"ifms/internal/mods/autocontrol/dal"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 设备类型信息
type DeviceTypeInfo struct {
	Trans             *util.Trans
	DeviceTypeInfoDAL *dal.DeviceTypeInfo
}

// Query device type infos from the data access object based on the provided parameters and options.
func (a *DeviceTypeInfo) Query(ctx context.Context, params schema.DeviceTypeInfoQueryParam) (*schema.DeviceTypeInfoQueryResult, error) {
	params.Pagination = true

	result, err := a.DeviceTypeInfoDAL.Query(ctx, params, schema.DeviceTypeInfoQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified device type info from the data access object.
func (a *DeviceTypeInfo) Get(ctx context.Context, id string) (*schema.DeviceTypeInfo, error) {
	deviceTypeInfo, err := a.DeviceTypeInfoDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if deviceTypeInfo == nil {
		return nil, errors.NotFound("", "Device type info not found")
	}
	return deviceTypeInfo, nil
}

// Create a new device type info in the data access object.
func (a *DeviceTypeInfo) Create(ctx context.Context, formItem *schema.DeviceTypeInfoForm) (*schema.DeviceTypeInfo, error) {
	deviceTypeInfo := &schema.DeviceTypeInfo{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(deviceTypeInfo); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceTypeInfoDAL.Create(ctx, deviceTypeInfo); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return deviceTypeInfo, nil
}

// Update the specified device type info in the data access object.
func (a *DeviceTypeInfo) Update(ctx context.Context, id string, formItem *schema.DeviceTypeInfoForm) error {
	deviceTypeInfo, err := a.DeviceTypeInfoDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if deviceTypeInfo == nil {
		return errors.NotFound("", "Device type info not found")
	}

	if err := formItem.FillTo(deviceTypeInfo); err != nil {
		return err
	}
	deviceTypeInfo.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceTypeInfoDAL.Update(ctx, deviceTypeInfo); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified device type info from the data access object.
func (a *DeviceTypeInfo) Delete(ctx context.Context, id string) error {
	exists, err := a.DeviceTypeInfoDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Device type info not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DeviceTypeInfoDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
