package dal

import (
	"context"

	"ifms/internal/consts"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get device control log storage instance
func GetDeviceControlLogDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.DeviceControlLog))
}

// 设备操作日志表
type DeviceControlLog struct {
	DB *gorm.DB
}

// Query device control logs from the database based on the provided parameters and options.
func (a *DeviceControlLog) Query(ctx context.Context, params schema.DeviceControlLogQueryParam, opts ...schema.DeviceControlLogQueryOptions) (*schema.DeviceControlLogQueryResult, error) {
	var opt schema.DeviceControlLogQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetDeviceControlLogDB(ctx, a.DB)

	if v := params.Type; v != nil {
		db = db.Where("type = ?", v)
	}
	if v := params.DeviceName; v != "" {
		db = db.Where("device_name LIKE ?", "%"+v+"%")
	}

	var list schema.DeviceControlLogs
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.DeviceControlLogQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified device control log from the database.
func (a *DeviceControlLog) Get(ctx context.Context, id string, opts ...schema.DeviceControlLogQueryOptions) (*schema.DeviceControlLog, error) {
	var opt schema.DeviceControlLogQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.DeviceControlLog)
	ok, err := util.FindOne(ctx, GetDeviceControlLogDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified device control log exists in the database.
func (a *DeviceControlLog) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetDeviceControlLogDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new device control log.
func (a *DeviceControlLog) Create(ctx context.Context, item *schema.DeviceControlLog) error {
	result := GetDeviceControlLogDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified device control log in the database.
func (a *DeviceControlLog) Update(ctx context.Context, item *schema.DeviceControlLog) error {
	result := GetDeviceControlLogDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified device control log from the database.
func (a *DeviceControlLog) Delete(ctx context.Context, id string) error {
	result := GetDeviceControlLogDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.DeviceControlLog))
	return errors.WithStack(result.Error)
}
