package dal

import (
	"context"

	"ifms/internal/consts"
	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get device fish pond storage instance
func GetDeviceFishPondDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.DeviceFishPond))
}

// 设备-鱼池关联表
type DeviceFishPond struct {
	DB *gorm.DB
}

// Query device fish ponds from the database based on the provided parameters and options.
func (a *DeviceFishPond) Query(ctx context.Context, params schema.DeviceFishPondQueryParam, opts ...schema.DeviceFishPondQueryOptions) (*schema.DeviceFishPondQueryResult, error) {
	var opt schema.DeviceFishPondQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetDeviceFishPondDB(ctx, a.DB)

	var list schema.DeviceFishPonds
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.DeviceFishPondQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified device fish pond from the database.
func (a *DeviceFishPond) Get(ctx context.Context, id string, opts ...schema.DeviceFishPondQueryOptions) (*schema.DeviceFishPond, error) {
	var opt schema.DeviceFishPondQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.DeviceFishPond)
	ok, err := util.FindOne(ctx, GetDeviceFishPondDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified device fish pond exists in the database.
func (a *DeviceFishPond) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetDeviceFishPondDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new device fish pond.
func (a *DeviceFishPond) Create(ctx context.Context, item *schema.DeviceFishPond) error {
	result := GetDeviceFishPondDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified device fish pond in the database.
func (a *DeviceFishPond) Update(ctx context.Context, item *schema.DeviceFishPond) error {
	result := GetDeviceFishPondDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified device fish pond from the database.
func (a *DeviceFishPond) Delete(ctx context.Context, id string) error {
	result := GetDeviceFishPondDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.DeviceFishPond))
	return errors.WithStack(result.Error)
}

func (a *DeviceFishPond) DeleteByDeviceID(ctx context.Context, deviceID int64) error {
	result := GetDeviceFishPondDB(ctx, a.DB).Where("device_id=?", deviceID).Delete(new(schema.DeviceFishPond))
	return errors.WithStack(result.Error)
}
