package api

import (
	"ifms/internal/mods/digital/biz"
	"ifms/internal/mods/digital/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 图纸管理
type Diagram struct {
	DiagramBIZ *biz.Diagram
}

// @Tags DiagramAPI
// @Security ApiKeyAuth
// @Summary Query diagram list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.Diagram}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/digital/diagrams [get]
func (a *Diagram) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.DiagramQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DiagramBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags DiagramAPI
// @Security ApiKeyAuth
// @Summary Get diagram record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.Diagram}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/digital/diagrams/{id} [get]
func (a *Diagram) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.DiagramBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags DiagramAPI
// @Security ApiKeyAuth
// @Summary Create diagram record
// @Param body body schema.DiagramForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.Diagram}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/digital/diagrams/create [post]
func (a *Diagram) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DiagramForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.DiagramBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result.ID)
}

// @Tags DiagramAPI
// @Security ApiKeyAuth
// @Summary Update diagram record by ID
// @Param id path string true "unique id"
// @Param body body schema.DiagramForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/digital/diagrams/update/{id} [post]
func (a *Diagram) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.DiagramForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.DiagramBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags DiagramAPI
// @Security ApiKeyAuth
// @Summary Delete diagram record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/digital/diagrams/delete [post]
func (a *Diagram) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.DiagramBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
