package biz

import (
	"context"
	"time"

	"ifms/internal/mods/digital/dal"
	"ifms/internal/mods/digital/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 图纸管理
type Diagram struct {
	Trans      *util.Trans
	DiagramDAL *dal.Diagram
}

// Query diagrams from the data access object based on the provided parameters and options.
func (a *Diagram) Query(ctx context.Context, params schema.DiagramQueryParam) (*schema.DiagramQueryResult, error) {
	params.Pagination = true

	result, err := a.DiagramDAL.Query(ctx, params, schema.DiagramQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified diagram from the data access object.
func (a *Diagram) Get(ctx context.Context, id string) (*schema.Diagram, error) {
	diagram, err := a.DiagramDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if diagram == nil {
		return nil, errors.NotFound("", "Diagram not found")
	}
	return diagram, nil
}

// Create a new diagram in the data access object.
func (a *Diagram) Create(ctx context.Context, formItem *schema.DiagramForm) (*schema.Diagram, error) {
	diagram := &schema.Diagram{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(diagram); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DiagramDAL.Create(ctx, diagram); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return diagram, nil
}

// Update the specified diagram in the data access object.
func (a *Diagram) Update(ctx context.Context, id string, formItem *schema.DiagramForm) error {
	diagram, err := a.DiagramDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if diagram == nil {
		return errors.NotFound("", "Diagram not found")
	}

	if err := formItem.FillTo(diagram); err != nil {
		return err
	}
	diagram.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DiagramDAL.Update(ctx, diagram); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified diagram from the data access object.
func (a *Diagram) Delete(ctx context.Context, id string) error {
	exists, err := a.DiagramDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Diagram not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.DiagramDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
