package dal

import (
	"context"

	"ifms/internal/mods/digital/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get diagram storage instance
func GetDiagramDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.Diagram))
}

// 图纸管理
type Diagram struct {
	DB *gorm.DB
}

// Query diagrams from the database based on the provided parameters and options.
func (a *Diagram) Query(ctx context.Context, params schema.DiagramQueryParam, opts ...schema.DiagramQueryOptions) (*schema.DiagramQueryResult, error) {
	var opt schema.DiagramQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	db := GetDiagramDB(ctx, a.DB)

	var list schema.Diagrams
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}
	return &schema.DiagramQueryResult{
		PageResult: pageResult,
		Data:       list,
	}, nil
}

// Get the specified diagram from the database.
func (a *Diagram) Get(ctx context.Context, id string, opts ...schema.DiagramQueryOptions) (*schema.Diagram, error) {
	var opt schema.DiagramQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.Diagram)
	ok, err := util.FindOne(ctx, GetDiagramDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified diagram exists in the database.
func (a *Diagram) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetDiagramDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new diagram.
func (a *Diagram) Create(ctx context.Context, item *schema.Diagram) error {
	result := GetDiagramDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified diagram in the database.
func (a *Diagram) Update(ctx context.Context, item *schema.Diagram) error {
	result := GetDiagramDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified diagram from the database.
func (a *Diagram) Delete(ctx context.Context, id string) error {
	result := GetDiagramDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.Diagram))
	return errors.WithStack(result.Error)
}
