package api

import (
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 环境监测设备管理
type EnvMonitorDevice struct {
	EnvMonitorDeviceBIZ *biz.EnvMonitorDevice
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDevice}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-devices [get]
func (a *EnvMonitorDevice) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDevice}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-devices/{id} [get]
func (a *EnvMonitorDevice) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device record
// @Param body body schema.EnvMonitorDeviceForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDevice}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-devices [post]
func (a *EnvMonitorDevice) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	result, err := a.EnvMonitorDeviceBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-devices/{id} [put]
func (a *EnvMonitorDevice) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.EnvMonitorDeviceBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-devices/{id} [delete]
func (a *EnvMonitorDevice) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceAPI
// @Security ApiKeyAuth
// @Summary Get device statistics
// @Success 200 {object} util.ResponseResult{data=[]schema.DeviceStatistics}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/autocontrol/devices/statistics [get]
func (a *EnvMonitorDevice) Statistics(c *gin.Context) {
	ctx := c.Request.Context()
	statistics, err := a.EnvMonitorDeviceBIZ.Statistics(ctx)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, statistics)
}
