package api

import (
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 环境监测设备-鱼池关联表
type EnvMonitorDeviceFishPond struct {
	EnvMonitorDeviceFishPondBIZ *biz.EnvMonitorDeviceFishPond
}

// @Tags EnvMonitorDeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device fish pond list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceFishPond}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-fish-ponds [get]
func (a *EnvMonitorDeviceFishPond) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceFishPondQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceFishPondBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device fish pond record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceFishPond}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-fish-ponds/{id} [get]
func (a *EnvMonitorDeviceFishPond) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceFishPondBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device fish pond record
// @Param body body schema.EnvMonitorDeviceFishPondForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceFishPond}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-fish-ponds [post]
func (a *EnvMonitorDeviceFishPond) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceFishPondForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	result, err := a.EnvMonitorDeviceFishPondBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device fish pond record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceFishPondForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-fish-ponds/{id} [put]
func (a *EnvMonitorDeviceFishPond) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceFishPondForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.EnvMonitorDeviceFishPondBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceFishPondAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device fish pond record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-fish-ponds/{id} [delete]
func (a *EnvMonitorDeviceFishPond) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceFishPondBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
