package api

import (
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 设备传感器表
type EnvMonitorDeviceSensor struct {
	EnvMonitorDeviceSensorBIZ *biz.EnvMonitorDeviceSensor
}

// @Tags EnvMonitorDeviceSensorAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device sensor list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceSensor}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-sensors [get]
func (a *EnvMonitorDeviceSensor) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceSensorQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceSensorBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceSensorAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device sensor record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceSensor}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-sensors/{id} [get]
func (a *EnvMonitorDeviceSensor) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceSensorBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceSensorAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device sensor record
// @Param body body schema.EnvMonitorDeviceSensorForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceSensor}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-sensors [post]
func (a *EnvMonitorDeviceSensor) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceSensorForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceSensorBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceSensorAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device sensor record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceSensorForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-sensors/{id} [put]
func (a *EnvMonitorDeviceSensor) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceSensorForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.EnvMonitorDeviceSensorBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceSensorAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device sensor record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-sensors/{id} [delete]
func (a *EnvMonitorDeviceSensor) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceSensorBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
