package api

import (
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 环境监测设备类型信息
type EnvMonitorDeviceTypeInfo struct {
	EnvMonitorDeviceTypeInfoBIZ *biz.EnvMonitorDeviceTypeInfo
}

// @Tags EnvMonitorDeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device type info list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceTypeInfo}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-type-infos [get]
func (a *EnvMonitorDeviceTypeInfo) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceTypeInfoQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceTypeInfoBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device type info record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceTypeInfo}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-type-infos/{id} [get]
func (a *EnvMonitorDeviceTypeInfo) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceTypeInfoBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device type info record
// @Param body body schema.EnvMonitorDeviceTypeInfoForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceTypeInfo}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-type-infos [post]
func (a *EnvMonitorDeviceTypeInfo) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceTypeInfoForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID

	result, err := a.EnvMonitorDeviceTypeInfoBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device type info record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceTypeInfoForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-type-infos/{id} [put]
func (a *EnvMonitorDeviceTypeInfo) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceTypeInfoForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.EnvMonitorDeviceTypeInfoBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceTypeInfoAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device type info record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-type-infos/{id} [delete]
func (a *EnvMonitorDeviceTypeInfo) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceTypeInfoBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
