package api

import (
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// 环境监测设备报警范围配置表
type EnvMonitorDeviceWarningConfig struct {
	EnvMonitorDeviceWarningConfigBIZ *biz.EnvMonitorDeviceWarningConfig
}

// @Tags EnvMonitorDeviceWarningConfigAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device warning config list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceWarningConfig}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-configs [get]
func (a *EnvMonitorDeviceWarningConfig) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceWarningConfigQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceWarningConfigBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceWarningConfigAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device warning config record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceWarningConfig}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-configs/{id} [get]
func (a *EnvMonitorDeviceWarningConfig) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceWarningConfigBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceWarningConfigAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device warning config record
// @Param body body schema.EnvMonitorDeviceWarningConfigForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceWarningConfig}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-configs [post]
func (a *EnvMonitorDeviceWarningConfig) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceWarningConfigForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	result, err := a.EnvMonitorDeviceWarningConfigBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceWarningConfigAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device warning config record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceWarningConfigForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-configs/{id} [put]
func (a *EnvMonitorDeviceWarningConfig) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceWarningConfigForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	err := a.EnvMonitorDeviceWarningConfigBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceWarningConfigAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device warning config record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-configs/{id} [delete]
func (a *EnvMonitorDeviceWarningConfig) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceWarningConfigBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
