package api

import (
	"fmt"
	"ifms/internal/mods/envmonitor/biz"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/util"
	"strconv"
	"time"

	"github.com/gin-gonic/gin"
	"github.com/spf13/cast"
)

// 环境监测设备报警日志
type EnvMonitorDeviceWarningLog struct {
	EnvMonitorDeviceWarningLogBIZ *biz.EnvMonitorDeviceWarningLog
}

// 导出
// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Export env monitor device warning log list
// @Param body body schema.EnvMonitorDeviceWarningLogQueryParam true "Request body"
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceWarningLogExport}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs/export [post]
func (a *EnvMonitorDeviceWarningLog) Export(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceWarningLogQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceWarningLogBIZ.All(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	// 生成excel
	title := []string{"序号", "设备名称", "设备类型", "关联鱼池/车间", "报警指标", "监测数据", "参考数值范围", "报警模式", "报警时间"}
	data := make([][]string, len(result))
	for i, item := range result {
		data[i] = []string{strconv.Itoa(i + 1), item.DeviceName, item.DeviceModel, item.RelatedPond, item.Type, fmt.Sprintf("%.2f", item.CurrentValue), item.StandardRange, cast.ToString(item.WarningType), item.CreatedAt.Format("2006-01-02 15:04:05")}
	}
	localFilePath := "./export" + cast.ToString(time.Now().Format("20060102150405")) + ".xlsx"
	objName := "export/excel/" + cast.ToString(time.Now().Format("20060102150405")) + ".xlsx"
	url, err := util.ExportDataToOSS(ctx, title, data, localFilePath, objName)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, schema.EnvMonitorDeviceWarningLogExport{
		Url: url,
	})
}

// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Query env monitor device warning log list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Success 200 {object} util.ResponseResult{data=[]schema.EnvMonitorDeviceWarningLog}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs [get]
func (a *EnvMonitorDeviceWarningLog) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.EnvMonitorDeviceWarningLogQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.EnvMonitorDeviceWarningLogBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Get env monitor device warning log record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceWarningLog}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs/{id} [get]
func (a *EnvMonitorDeviceWarningLog) Get(c *gin.Context) {
	ctx := c.Request.Context()
	item, err := a.EnvMonitorDeviceWarningLogBIZ.Get(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, item)
}

// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Create env monitor device warning log record
// @Param body body schema.EnvMonitorDeviceWarningLogForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.EnvMonitorDeviceWarningLog}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs [post]
func (a *EnvMonitorDeviceWarningLog) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceWarningLogForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID

	result, err := a.EnvMonitorDeviceWarningLogBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResSuccess(c, result)
}

// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Update env monitor device warning log record by ID
// @Param id path string true "unique id"
// @Param body body schema.EnvMonitorDeviceWarningLogForm true "Request body"
// @Success 200 {object} util.ResponseResult
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs/{id} [put]
func (a *EnvMonitorDeviceWarningLog) Update(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.EnvMonitorDeviceWarningLogForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}

	err := a.EnvMonitorDeviceWarningLogBIZ.Update(ctx, c.Param("id"), item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags EnvMonitorDeviceWarningLogAPI
// @Security ApiKeyAuth
// @Summary Delete env monitor device warning log record by ID
// @Param id path string true "unique id"
// @Success 200 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/envmonitor/env-monitor-device-warning-logs/{id} [delete]
func (a *EnvMonitorDeviceWarningLog) Delete(c *gin.Context) {
	ctx := c.Request.Context()
	err := a.EnvMonitorDeviceWarningLogBIZ.Delete(ctx, c.Param("id"))
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}
