package biz

import (
	"context"
	"time"

	"ifms/internal/consts"
	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"
)

// 环境监测设备管理
type EnvMonitorDevice struct {
	Trans                       *util.Trans
	EnvMonitorDeviceDAL         *dal.EnvMonitorDevice
	EnvMonitorDeviceFishPondDAL *dal.EnvMonitorDeviceFishPond
	EnvMonitorDeviceSensorDAL   *dal.EnvMonitorDeviceSensor
}

// Query env monitor devices from the data access object based on the provided parameters and options.
func (a *EnvMonitorDevice) Query(ctx context.Context, params schema.EnvMonitorDeviceQueryParam) (*schema.EnvMonitorDeviceQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceDAL.Query(ctx, params, schema.EnvMonitorDeviceQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device from the data access object.
func (a *EnvMonitorDevice) Get(ctx context.Context, id string) (*schema.EnvMonitorDevice, error) {
	envMonitorDevice, err := a.EnvMonitorDeviceDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDevice == nil {
		return nil, errors.NotFound("", "Env monitor device not found")
	}
	return envMonitorDevice, nil
}

// Create a new env monitor device in the data access object.
func (a *EnvMonitorDevice) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceForm) (*schema.EnvMonitorDevice, error) {
	envMonitorDevice := &schema.EnvMonitorDevice{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDevice); err != nil {
		return nil, err
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceDAL.Create(ctx, envMonitorDevice); err != nil {
			return err
		}
		// 关联鱼池
		if len(formItem.EnvMonitorDeviceDeviceFishPonds) > 0 {
			for _, fishPond := range formItem.EnvMonitorDeviceDeviceFishPonds {
				deviceFishPond := &schema.EnvMonitorDeviceFishPond{
					EnvMonitorDeviceID: envMonitorDevice.ID,
					FishPondID:         fishPond.FishPondID,
					FishPondName:       fishPond.FishPondName,
					TenantID:           envMonitorDevice.TenantID,
				}
				if err := a.EnvMonitorDeviceFishPondDAL.Create(ctx, deviceFishPond); err != nil {
					return err
				}
			}
		}
		// 关联传感器
		if len(formItem.EnvMonitorDeviceDeviceSensors) > 0 {
			for _, sensor := range formItem.EnvMonitorDeviceDeviceSensors {
				deviceSensor := &schema.EnvMonitorDeviceSensor{
					EnvMonitorDeviceID: envMonitorDevice.ID,
					Name:               sensor.Name,
					Status:             sensor.Status,
					CurrentValue:       sensor.CurrentValue,
					TenantID:           envMonitorDevice.TenantID,
					Type:               sensor.Type,
				}
				if err := a.EnvMonitorDeviceSensorDAL.Create(ctx, deviceSensor); err != nil {
					return err
				}
			}
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDevice, nil
}

// Update the specified env monitor device in the data access object.
func (a *EnvMonitorDevice) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceForm) error {
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)

	envMonitorDevice, err := a.EnvMonitorDeviceDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDevice == nil {
		return errors.NotFound("", "Env monitor device not found")
	}

	if err := formItem.FillTo(envMonitorDevice); err != nil {
		return err
	}
	envMonitorDevice.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceDAL.Update(ctx, envMonitorDevice); err != nil {
			return err
		}
		if err := a.EnvMonitorDeviceFishPondDAL.DeleteByDeviceID(ctx, envMonitorDevice.ID); err != nil {
			return err
		}
		if len(formItem.EnvMonitorDeviceDeviceFishPonds) > 0 {
			for _, fishPond := range formItem.EnvMonitorDeviceDeviceFishPonds {
				deviceFishPond := &schema.EnvMonitorDeviceFishPond{
					EnvMonitorDeviceID: envMonitorDevice.ID,
					FishPondID:         fishPond.FishPondID,
					FishPondName:       fishPond.FishPondName,
					TenantID:           envMonitorDevice.TenantID,
				}
				if err := a.EnvMonitorDeviceFishPondDAL.Create(ctx, deviceFishPond); err != nil {
					return err
				}
			}
		}

		// 关联传感器
		if err := a.EnvMonitorDeviceSensorDAL.DeleteByDeviceID(ctx, envMonitorDevice.ID); err != nil {
			return err
		}
		if len(formItem.EnvMonitorDeviceDeviceSensors) > 0 {
			for _, sensor := range formItem.EnvMonitorDeviceDeviceSensors {
				deviceSensor := &schema.EnvMonitorDeviceSensor{
					EnvMonitorDeviceID: envMonitorDevice.ID,
					Name:               sensor.Name,
					Status:             sensor.Status,
					CurrentValue:       sensor.CurrentValue,
					TenantID:           envMonitorDevice.TenantID,
					Type:               sensor.Type,
				}
				if err := a.EnvMonitorDeviceSensorDAL.Create(ctx, deviceSensor); err != nil {
					return err
				}
			}
		}
		return nil
	})
}

// Delete the specified env monitor device from the data access object.
func (a *EnvMonitorDevice) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}

func (a *EnvMonitorDevice) Statistics(ctx context.Context) ([]schema.DeviceStatistics, error) {
	statistics, err := a.EnvMonitorDeviceDAL.Statistics(ctx)
	if err != nil {
		return nil, err
	}
	return statistics, nil
}
