package biz

import (
	"context"
	"time"

	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 环境监测设备-鱼池关联表
type EnvMonitorDeviceFishPond struct {
	Trans                       *util.Trans
	EnvMonitorDeviceFishPondDAL *dal.EnvMonitorDeviceFishPond
}

// Query env monitor device fish ponds from the data access object based on the provided parameters and options.
func (a *EnvMonitorDeviceFishPond) Query(ctx context.Context, params schema.EnvMonitorDeviceFishPondQueryParam) (*schema.EnvMonitorDeviceFishPondQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceFishPondDAL.Query(ctx, params, schema.EnvMonitorDeviceFishPondQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device fish pond from the data access object.
func (a *EnvMonitorDeviceFishPond) Get(ctx context.Context, id string) (*schema.EnvMonitorDeviceFishPond, error) {
	envMonitorDeviceFishPond, err := a.EnvMonitorDeviceFishPondDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDeviceFishPond == nil {
		return nil, errors.NotFound("", "Env monitor device fish pond not found")
	}
	return envMonitorDeviceFishPond, nil
}

// Create a new env monitor device fish pond in the data access object.
func (a *EnvMonitorDeviceFishPond) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceFishPondForm) (*schema.EnvMonitorDeviceFishPond, error) {
	envMonitorDeviceFishPond := &schema.EnvMonitorDeviceFishPond{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDeviceFishPond); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceFishPondDAL.Create(ctx, envMonitorDeviceFishPond); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDeviceFishPond, nil
}

// Update the specified env monitor device fish pond in the data access object.
func (a *EnvMonitorDeviceFishPond) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceFishPondForm) error {
	envMonitorDeviceFishPond, err := a.EnvMonitorDeviceFishPondDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDeviceFishPond == nil {
		return errors.NotFound("", "Env monitor device fish pond not found")
	}

	if err := formItem.FillTo(envMonitorDeviceFishPond); err != nil {
		return err
	}
	envMonitorDeviceFishPond.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceFishPondDAL.Update(ctx, envMonitorDeviceFishPond); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified env monitor device fish pond from the data access object.
func (a *EnvMonitorDeviceFishPond) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceFishPondDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device fish pond not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceFishPondDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
