package biz

import (
	"context"
	"time"

	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 设备传感器表
type EnvMonitorDeviceSensor struct {
	Trans                     *util.Trans
	EnvMonitorDeviceSensorDAL *dal.EnvMonitorDeviceSensor
}

// Query env monitor device sensors from the data access object based on the provided parameters and options.
func (a *EnvMonitorDeviceSensor) Query(ctx context.Context, params schema.EnvMonitorDeviceSensorQueryParam) (*schema.EnvMonitorDeviceSensorQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceSensorDAL.Query(ctx, params, schema.EnvMonitorDeviceSensorQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device sensor from the data access object.
func (a *EnvMonitorDeviceSensor) Get(ctx context.Context, id string) (*schema.EnvMonitorDeviceSensor, error) {
	envMonitorDeviceSensor, err := a.EnvMonitorDeviceSensorDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDeviceSensor == nil {
		return nil, errors.NotFound("", "Env monitor device sensor not found")
	}
	return envMonitorDeviceSensor, nil
}

// Create a new env monitor device sensor in the data access object.
func (a *EnvMonitorDeviceSensor) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceSensorForm) (*schema.EnvMonitorDeviceSensor, error) {
	envMonitorDeviceSensor := &schema.EnvMonitorDeviceSensor{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDeviceSensor); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceSensorDAL.Create(ctx, envMonitorDeviceSensor); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDeviceSensor, nil
}

// Update the specified env monitor device sensor in the data access object.
func (a *EnvMonitorDeviceSensor) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceSensorForm) error {
	envMonitorDeviceSensor, err := a.EnvMonitorDeviceSensorDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDeviceSensor == nil {
		return errors.NotFound("", "Env monitor device sensor not found")
	}

	if err := formItem.FillTo(envMonitorDeviceSensor); err != nil {
		return err
	}
	envMonitorDeviceSensor.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceSensorDAL.Update(ctx, envMonitorDeviceSensor); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified env monitor device sensor from the data access object.
func (a *EnvMonitorDeviceSensor) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceSensorDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device sensor not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceSensorDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
