package biz

import (
	"context"
	"time"

	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 环境监测设备类型信息
type EnvMonitorDeviceTypeInfo struct {
	Trans                       *util.Trans
	EnvMonitorDeviceTypeInfoDAL *dal.EnvMonitorDeviceTypeInfo
}

// Query env monitor device type infos from the data access object based on the provided parameters and options.
func (a *EnvMonitorDeviceTypeInfo) Query(ctx context.Context, params schema.EnvMonitorDeviceTypeInfoQueryParam) (*schema.EnvMonitorDeviceTypeInfoQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceTypeInfoDAL.Query(ctx, params, schema.EnvMonitorDeviceTypeInfoQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device type info from the data access object.
func (a *EnvMonitorDeviceTypeInfo) Get(ctx context.Context, id string) (*schema.EnvMonitorDeviceTypeInfo, error) {
	envMonitorDeviceTypeInfo, err := a.EnvMonitorDeviceTypeInfoDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDeviceTypeInfo == nil {
		return nil, errors.NotFound("", "Env monitor device type info not found")
	}
	return envMonitorDeviceTypeInfo, nil
}

// Create a new env monitor device type info in the data access object.
func (a *EnvMonitorDeviceTypeInfo) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceTypeInfoForm) (*schema.EnvMonitorDeviceTypeInfo, error) {
	envMonitorDeviceTypeInfo := &schema.EnvMonitorDeviceTypeInfo{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDeviceTypeInfo); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceTypeInfoDAL.Create(ctx, envMonitorDeviceTypeInfo); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDeviceTypeInfo, nil
}

// Update the specified env monitor device type info in the data access object.
func (a *EnvMonitorDeviceTypeInfo) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceTypeInfoForm) error {
	envMonitorDeviceTypeInfo, err := a.EnvMonitorDeviceTypeInfoDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDeviceTypeInfo == nil {
		return errors.NotFound("", "Env monitor device type info not found")
	}

	if err := formItem.FillTo(envMonitorDeviceTypeInfo); err != nil {
		return err
	}
	envMonitorDeviceTypeInfo.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceTypeInfoDAL.Update(ctx, envMonitorDeviceTypeInfo); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified env monitor device type info from the data access object.
func (a *EnvMonitorDeviceTypeInfo) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceTypeInfoDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device type info not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceTypeInfoDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
