package biz

import (
	"context"
	"time"

	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 环境监测设备报警范围配置表
type EnvMonitorDeviceWarningConfig struct {
	Trans                            *util.Trans
	EnvMonitorDeviceWarningConfigDAL *dal.EnvMonitorDeviceWarningConfig
}

// Query env monitor device warning configs from the data access object based on the provided parameters and options.
func (a *EnvMonitorDeviceWarningConfig) Query(ctx context.Context, params schema.EnvMonitorDeviceWarningConfigQueryParam) (*schema.EnvMonitorDeviceWarningConfigQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceWarningConfigDAL.Query(ctx, params, schema.EnvMonitorDeviceWarningConfigQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device warning config from the data access object.
func (a *EnvMonitorDeviceWarningConfig) Get(ctx context.Context, id string) (*schema.EnvMonitorDeviceWarningConfig, error) {
	envMonitorDeviceWarningConfig, err := a.EnvMonitorDeviceWarningConfigDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDeviceWarningConfig == nil {
		return nil, errors.NotFound("", "Env monitor device warning config not found")
	}
	return envMonitorDeviceWarningConfig, nil
}

// Create a new env monitor device warning config in the data access object.
func (a *EnvMonitorDeviceWarningConfig) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceWarningConfigForm) (*schema.EnvMonitorDeviceWarningConfig, error) {
	envMonitorDeviceWarningConfig := &schema.EnvMonitorDeviceWarningConfig{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDeviceWarningConfig); err != nil {
		return nil, err
	}
	// 如果该租户已经存在报警范围配置，则不允许新增
	exists, err := a.EnvMonitorDeviceWarningConfigDAL.ExistsByTenantID(ctx, envMonitorDeviceWarningConfig.TenantID, envMonitorDeviceWarningConfig.Type)
	if err != nil {
		return nil, err
	} else if exists {
		return nil, errors.BadRequest("", "已经存在报警范围配置，不允许新增")
	}

	err = a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningConfigDAL.Create(ctx, envMonitorDeviceWarningConfig); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDeviceWarningConfig, nil
}

// Update the specified env monitor device warning config in the data access object.
func (a *EnvMonitorDeviceWarningConfig) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceWarningConfigForm) error {
	envMonitorDeviceWarningConfig, err := a.EnvMonitorDeviceWarningConfigDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDeviceWarningConfig == nil {
		return errors.NotFound("", "Env monitor device warning config not found")
	}

	if err := formItem.FillTo(envMonitorDeviceWarningConfig); err != nil {
		return err
	}
	envMonitorDeviceWarningConfig.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningConfigDAL.Update(ctx, envMonitorDeviceWarningConfig); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified env monitor device warning config from the data access object.
func (a *EnvMonitorDeviceWarningConfig) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceWarningConfigDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device warning config not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningConfigDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
