package biz

import (
	"context"
	"time"

	"ifms/internal/mods/envmonitor/dal"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

// 环境监测设备报警日志
type EnvMonitorDeviceWarningLog struct {
	Trans                         *util.Trans
	EnvMonitorDeviceWarningLogDAL *dal.EnvMonitorDeviceWarningLog
}

func (a *EnvMonitorDeviceWarningLog) All(ctx context.Context, params schema.EnvMonitorDeviceWarningLogQueryParam) ([]*schema.EnvMonitorDeviceWarningLog, error) {
	result, err := a.EnvMonitorDeviceWarningLogDAL.QueryAll(ctx, params)
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Query env monitor device warning logs from the data access object based on the provided parameters and options.
func (a *EnvMonitorDeviceWarningLog) Query(ctx context.Context, params schema.EnvMonitorDeviceWarningLogQueryParam) (*schema.EnvMonitorDeviceWarningLogQueryResult, error) {
	params.Pagination = true

	result, err := a.EnvMonitorDeviceWarningLogDAL.Query(ctx, params, schema.EnvMonitorDeviceWarningLogQueryOptions{
		QueryOptions: util.QueryOptions{
			OrderFields: []util.OrderByParam{
				{Field: "created_at", Direction: util.DESC},
			},
		},
	})
	if err != nil {
		return nil, err
	}
	return result, nil
}

// Get the specified env monitor device warning log from the data access object.
func (a *EnvMonitorDeviceWarningLog) Get(ctx context.Context, id string) (*schema.EnvMonitorDeviceWarningLog, error) {
	envMonitorDeviceWarningLog, err := a.EnvMonitorDeviceWarningLogDAL.Get(ctx, id)
	if err != nil {
		return nil, err
	} else if envMonitorDeviceWarningLog == nil {
		return nil, errors.NotFound("", "Env monitor device warning log not found")
	}
	return envMonitorDeviceWarningLog, nil
}

// Create a new env monitor device warning log in the data access object.
func (a *EnvMonitorDeviceWarningLog) Create(ctx context.Context, formItem *schema.EnvMonitorDeviceWarningLogForm) (*schema.EnvMonitorDeviceWarningLog, error) {
	envMonitorDeviceWarningLog := &schema.EnvMonitorDeviceWarningLog{
		CreatedAt: time.Now(),
	}

	if err := formItem.FillTo(envMonitorDeviceWarningLog); err != nil {
		return nil, err
	}

	err := a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningLogDAL.Create(ctx, envMonitorDeviceWarningLog); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		return nil, err
	}
	return envMonitorDeviceWarningLog, nil
}

// Update the specified env monitor device warning log in the data access object.
func (a *EnvMonitorDeviceWarningLog) Update(ctx context.Context, id string, formItem *schema.EnvMonitorDeviceWarningLogForm) error {
	envMonitorDeviceWarningLog, err := a.EnvMonitorDeviceWarningLogDAL.Get(ctx, id)
	if err != nil {
		return err
	} else if envMonitorDeviceWarningLog == nil {
		return errors.NotFound("", "Env monitor device warning log not found")
	}

	if err := formItem.FillTo(envMonitorDeviceWarningLog); err != nil {
		return err
	}
	envMonitorDeviceWarningLog.UpdatedAt = time.Now()

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningLogDAL.Update(ctx, envMonitorDeviceWarningLog); err != nil {
			return err
		}
		return nil
	})
}

// Delete the specified env monitor device warning log from the data access object.
func (a *EnvMonitorDeviceWarningLog) Delete(ctx context.Context, id string) error {
	exists, err := a.EnvMonitorDeviceWarningLogDAL.Exists(ctx, id)
	if err != nil {
		return err
	} else if !exists {
		return errors.NotFound("", "Env monitor device warning log not found")
	}

	return a.Trans.Exec(ctx, func(ctx context.Context) error {
		if err := a.EnvMonitorDeviceWarningLogDAL.Delete(ctx, id); err != nil {
			return err
		}
		return nil
	})
}
