package dal

import (
	"context"
	"strings"

	"ifms/internal/consts"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device storage instance
func GetEnvMonitorDeviceDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDevice))
}

// 环境监测设备管理
type EnvMonitorDevice struct {
	DB *gorm.DB
}

// Query env monitor devices from the database based on the provided parameters and options.
func (a *EnvMonitorDevice) Query(ctx context.Context, params schema.EnvMonitorDeviceQueryParam, opts ...schema.EnvMonitorDeviceQueryOptions) (*schema.EnvMonitorDeviceQueryResult, error) {
	var opt schema.EnvMonitorDeviceQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)

	db := GetEnvMonitorDeviceDB(ctx, a.DB)
	db = db.Preload("EnvMonitorDeviceFishPond").Preload("EnvMonitorDeviceSensor")
	if params.DeviceName != "" {
		db = db.Where("name like ?", "%"+params.DeviceName+"%")
	}
	if params.Type != nil && *params.Type != 0 {
		db = db.Where("type=?", *params.Type)
	}
	if params.WorkshopId != "" {
		db = db.Where("workshop_id in (?)", strings.Split(params.WorkshopId, ","))
	}
	var list schema.EnvMonitorDevices
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device from the database.
func (a *EnvMonitorDevice) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceQueryOptions) (*schema.EnvMonitorDevice, error) {
	var opt schema.EnvMonitorDeviceQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	item := new(schema.EnvMonitorDevice)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device exists in the database.
func (a *EnvMonitorDevice) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device.
func (a *EnvMonitorDevice) Create(ctx context.Context, item *schema.EnvMonitorDevice) error {
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	result := GetEnvMonitorDeviceDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device in the database.
func (a *EnvMonitorDevice) Update(ctx context.Context, item *schema.EnvMonitorDevice) error {
	result := GetEnvMonitorDeviceDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device from the database.
func (a *EnvMonitorDevice) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDevice))
	return errors.WithStack(result.Error)
}

func (a *EnvMonitorDevice) Statistics(ctx context.Context) ([]schema.DeviceStatistics, error) {
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)

	// 统计不同的设备类型数量以及在线数量(status=true)
	var statistics []schema.DeviceStatistics
	//
	// EnvMonitorDeviceSensorTypeWaterTemperature = 1 // 水温传感器
	// EnvMonitorDeviceSensorTypeDissolvedOxygen  = 2 // 溶解氧传感器
	// EnvMonitorDeviceSensorTypeAmmonia          = 3 // 氨氮传感器
	// EnvMonitorDeviceSensorTypePH               = 4 // PH传感器
	// EnvMonitorDeviceSensorTypeConductivity     = 5 // 电导率传感器
	// EnvMonitorDeviceSensorTypeVideo            = 6 // 视频监控
	types := []int{schema.EnvMonitorDeviceSensorTypeWaterTemperature, schema.EnvMonitorDeviceSensorTypeDissolvedOxygen, schema.EnvMonitorDeviceSensorTypeAmmonia, schema.EnvMonitorDeviceSensorTypePH, schema.EnvMonitorDeviceSensorTypeConductivity, schema.EnvMonitorDeviceSensorTypeVideo}
	for _, v := range types {
		// 统计不同设备类型总数
		total := int64(0)
		GetEnvMonitorDeviceDB(ctx, a.DB).Model(&schema.EnvMonitorDeviceSensor{}).Where("type=?", v).Count(&total)
		// 统计不同设备类型在线数量
		online := int64(0)
		GetEnvMonitorDeviceDB(ctx, a.DB).Model(&schema.EnvMonitorDeviceSensor{}).Where("type=?", v).Where("status=?", true).Count(&online)
		statistics = append(statistics, schema.DeviceStatistics{
			Type:    v,
			Total:   int(total),
			Online:  int(online),
			Offline: int(total - online),
		})
		if total > 0 {
			statistics[len(statistics)-1].Rate = int(online * 100 / total)
		}
	}
	return statistics, nil
}
