package dal

import (
	"context"

	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device sensor storage instance
func GetEnvMonitorDeviceSensorDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDeviceSensor))
}

// 设备传感器表
type EnvMonitorDeviceSensor struct {
	DB *gorm.DB
}

// Query env monitor device sensors from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceSensor) Query(ctx context.Context, params schema.EnvMonitorDeviceSensorQueryParam, opts ...schema.EnvMonitorDeviceSensorQueryOptions) (*schema.EnvMonitorDeviceSensorQueryResult, error) {
	var opt schema.EnvMonitorDeviceSensorQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	db := GetEnvMonitorDeviceSensorDB(ctx, a.DB)

	var list schema.EnvMonitorDeviceSensors
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceSensorQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device sensor from the database.
func (a *EnvMonitorDeviceSensor) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceSensorQueryOptions) (*schema.EnvMonitorDeviceSensor, error) {
	var opt schema.EnvMonitorDeviceSensorQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.EnvMonitorDeviceSensor)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceSensorDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device sensor exists in the database.
func (a *EnvMonitorDeviceSensor) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceSensorDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device sensor.
func (a *EnvMonitorDeviceSensor) Create(ctx context.Context, item *schema.EnvMonitorDeviceSensor) error {
	result := GetEnvMonitorDeviceSensorDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device sensor in the database.
func (a *EnvMonitorDeviceSensor) Update(ctx context.Context, item *schema.EnvMonitorDeviceSensor) error {
	result := GetEnvMonitorDeviceSensorDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device sensor from the database.
func (a *EnvMonitorDeviceSensor) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceSensorDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDeviceSensor))
	return errors.WithStack(result.Error)
}

func (a *EnvMonitorDeviceSensor) DeleteByDeviceID(ctx context.Context, deviceID int64) error {
	result := GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("env_monitor_device_id=?", deviceID).Delete(new(schema.EnvMonitorDeviceFishPond))
	return errors.WithStack(result.Error)
}
