package dal

import (
	"context"

	"ifms/internal/consts"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device warning config storage instance
func GetEnvMonitorDeviceWarningConfigDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDeviceWarningConfig))
}

// 环境监测设备报警范围配置表
type EnvMonitorDeviceWarningConfig struct {
	DB *gorm.DB
}

// Query env monitor device warning configs from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceWarningConfig) Query(ctx context.Context, params schema.EnvMonitorDeviceWarningConfigQueryParam, opts ...schema.EnvMonitorDeviceWarningConfigQueryOptions) (*schema.EnvMonitorDeviceWarningConfigQueryResult, error) {
	var opt schema.EnvMonitorDeviceWarningConfigQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB)
	if params.Type != 0 {
		db = db.Where("type = ?", params.Type)
	}
	var list schema.EnvMonitorDeviceWarningConfigs
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceWarningConfigQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device warning config from the database.
func (a *EnvMonitorDeviceWarningConfig) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceWarningConfigQueryOptions) (*schema.EnvMonitorDeviceWarningConfig, error) {
	var opt schema.EnvMonitorDeviceWarningConfigQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.EnvMonitorDeviceWarningConfig)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device warning config exists in the database.
func (a *EnvMonitorDeviceWarningConfig) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

func (a *EnvMonitorDeviceWarningConfig) ExistsByTenantID(ctx context.Context, tenantID string, typeT int) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Where("tenant_id=? and type=?", tenantID, typeT))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device warning config.
func (a *EnvMonitorDeviceWarningConfig) Create(ctx context.Context, item *schema.EnvMonitorDeviceWarningConfig) error {
	result := GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device warning config in the database.
func (a *EnvMonitorDeviceWarningConfig) Update(ctx context.Context, item *schema.EnvMonitorDeviceWarningConfig) error {
	result := GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device warning config from the database.
func (a *EnvMonitorDeviceWarningConfig) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceWarningConfigDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDeviceWarningConfig))
	return errors.WithStack(result.Error)
}
