package dal

import (
	"context"
	"fmt"
	"strings"

	"ifms/internal/consts"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device warning log storage instance
func GetEnvMonitorDeviceWarningLogDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDeviceWarningLog))
}

// 环境监测设备报警日志
type EnvMonitorDeviceWarningLog struct {
	DB *gorm.DB
}

// Query env monitor device warning logs from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceWarningLog) QueryAll(ctx context.Context, params schema.EnvMonitorDeviceWarningLogQueryParam) ([]*schema.EnvMonitorDeviceWarningLog, error) {
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetEnvMonitorDeviceWarningLogDB(ctx, a.DB)
	if params.DeviceName != "" {
		db = db.Where("device_name like ?", "%"+params.DeviceName+"%")
	}
	if params.StartTime != "" {
		db = db.Where("created_at >= ?", params.StartTime)
	}
	if params.EndTime != "" {
		db = db.Where("created_at <= ?", params.EndTime)
	}

	if params.IsOK != nil {
		db = db.Where("is_ok = ?", *params.IsOK)
	}

	if params.Ids != "" {
		db = db.Where("id in (?)", strings.Split(params.Ids, ","))
	}

	if params.Type != "" {
		db = db.Where("type = ?", params.Type)
	}

	var list []*schema.EnvMonitorDeviceWarningLog
	err := db.Find(&list).Error
	if err != nil {
		return nil, errors.WithStack(err)
	}

	return list, nil
}

// Query env monitor device warning logs from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceWarningLog) Query(ctx context.Context, params schema.EnvMonitorDeviceWarningLogQueryParam, opts ...schema.EnvMonitorDeviceWarningLogQueryOptions) (*schema.EnvMonitorDeviceWarningLogQueryResult, error) {
	var opt schema.EnvMonitorDeviceWarningLogQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetEnvMonitorDeviceWarningLogDB(ctx, a.DB)
	if params.DeviceName != "" {
		db = db.Where("device_name like ?", "%"+params.DeviceName+"%")
	}
	if params.StartTime != "" {
		db = db.Where("created_at >= ?", params.StartTime)
	}
	if params.EndTime != "" {
		db = db.Where("created_at <= ?", params.EndTime)
	}
	if params.IsOK != nil {
		db = db.Where("is_ok = ?", *params.IsOK)
	}

	if params.Ids != "" {
		db = db.Where("id in (?)", strings.Split(params.Ids, ","))
	}

	if params.Type != "" {
		db = db.Where("type = ?", params.Type)
	}

	var list schema.EnvMonitorDeviceWarningLogs
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceWarningLogQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device warning log from the database.
func (a *EnvMonitorDeviceWarningLog) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceWarningLogQueryOptions) (*schema.EnvMonitorDeviceWarningLog, error) {
	var opt schema.EnvMonitorDeviceWarningLogQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.EnvMonitorDeviceWarningLog)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceWarningLogDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device warning log exists in the database.
func (a *EnvMonitorDeviceWarningLog) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceWarningLogDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device warning log.
func (a *EnvMonitorDeviceWarningLog) Create(ctx context.Context, item *schema.EnvMonitorDeviceWarningLog) error {
	// 查询当前租户设置的值
	tenantID := item.TenantID
	envMonitorDeviceWarningConfig := &schema.EnvMonitorDeviceWarningConfig{}
	err := a.DB.Model(new(schema.EnvMonitorDeviceWarningConfig)).Where("tenant_id=?", tenantID).First(envMonitorDeviceWarningConfig).Error
	if err != nil {
		return errors.WithStack(err)
	}
	if envMonitorDeviceWarningConfig.ID == 0 {
		return errors.New("env monitor device warning config not found", "env monitor device warning config not found", 400)
	}
	// 查询设备传感器
	envMonitorDeviceSensor := &schema.EnvMonitorDeviceSensor{}
	err = a.DB.Model(new(schema.EnvMonitorDeviceSensor)).Where("id=?", item.EnvMonitorDeviceSensorID).First(envMonitorDeviceSensor).Error
	if err != nil {
		return errors.WithStack(err)
	}
	if envMonitorDeviceSensor.ID == 0 {
		return errors.New("env monitor device sensor not found", "env monitor device sensor not found", 400)
	}
	item.IsOk = true
	typeStr := ""
	switch envMonitorDeviceSensor.Type {
	case 1:
		typeStr = "水温"
	case 2:
		typeStr = "溶解氧"
	case 3:
		typeStr = "氨氮"
	case 4:
		typeStr = "PH值"
	case 5:
		typeStr = "电导率"
	case 6:
		typeStr = "视频监控"
	}
	item.Type = typeStr
	// 水温
	if item.Type == "水温" {
		waterLower := envMonitorDeviceWarningConfig.WaterTempMin
		waterUpper := envMonitorDeviceWarningConfig.WaterTempMax
		item.StandardRange = fmt.Sprintf("%.2f-%.2f", waterLower, waterUpper)
		if item.CurrentValue < waterLower || item.CurrentValue > waterUpper {
			item.IsOk = false
			if item.CurrentValue > waterUpper {
				item.WarningType = 1
			}
			if item.CurrentValue < waterLower {
				item.WarningType = 2
			}
		}

	}

	// 电导率
	if item.Type == "电导率" {
		conductivityLower := envMonitorDeviceWarningConfig.ConductivityMin
		conductivityUpper := envMonitorDeviceWarningConfig.ConductivityMax
		// 数值保留两位小数
		item.StandardRange = fmt.Sprintf("%.2f-%.2f", conductivityLower, conductivityUpper)
		if item.CurrentValue < conductivityLower || item.CurrentValue > conductivityUpper {
			item.IsOk = false
			if item.CurrentValue > conductivityUpper {
				item.WarningType = 1
			}
			if item.CurrentValue < conductivityLower {
				item.WarningType = 2
			}
		}
	}
	// 氨氮
	if item.Type == "氨氮" {
		ammoniaNitrogenLower := envMonitorDeviceWarningConfig.AmmoniaNitrogenMin
		ammoniaNitrogenUpper := envMonitorDeviceWarningConfig.AmmoniaNitrogenMax
		item.StandardRange = fmt.Sprintf("%.2f-%.2f", ammoniaNitrogenLower, ammoniaNitrogenUpper)
		if item.CurrentValue < ammoniaNitrogenLower || item.CurrentValue > ammoniaNitrogenUpper {
			item.IsOk = false
			if item.CurrentValue > ammoniaNitrogenUpper {
				item.WarningType = 1
			}
			if item.CurrentValue < ammoniaNitrogenLower {
				item.WarningType = 2
			}
		}
	}

	// 溶解氧
	if item.Type == "溶解氧" {
		dissolvedOxygenLower := envMonitorDeviceWarningConfig.DissolvedOxygenMin
		dissolvedOxygenUpper := envMonitorDeviceWarningConfig.DissolvedOxygenMax
		item.StandardRange = fmt.Sprintf("%.2f-%.2f", dissolvedOxygenLower, dissolvedOxygenUpper)
		if item.CurrentValue < dissolvedOxygenLower || item.CurrentValue > dissolvedOxygenUpper {
			item.IsOk = false
			if item.CurrentValue > dissolvedOxygenUpper {
				item.WarningType = 1
			}
			if item.CurrentValue < dissolvedOxygenLower {
				item.WarningType = 2
			}
		}
	}
	// PH值
	if item.Type == "PH值" {
		phLower := envMonitorDeviceWarningConfig.PHMin
		phUpper := envMonitorDeviceWarningConfig.PHMax
		item.StandardRange = fmt.Sprintf("%.2f-%.2f", phLower, phUpper)
		if item.CurrentValue < phLower || item.CurrentValue > phUpper {
			item.IsOk = false
			if item.CurrentValue > phUpper {
				item.WarningType = 1
			}
			if item.CurrentValue < phLower {
				item.WarningType = 2
			}
		}
	}

	result := GetEnvMonitorDeviceWarningLogDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device warning log in the database.
func (a *EnvMonitorDeviceWarningLog) Update(ctx context.Context, item *schema.EnvMonitorDeviceWarningLog) error {
	result := GetEnvMonitorDeviceWarningLogDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device warning log from the database.
func (a *EnvMonitorDeviceWarningLog) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceWarningLogDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDeviceWarningLog))
	return errors.WithStack(result.Error)
}
