package schema

import (
	"time"

	"ifms/pkg/util"
)

// 设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
const (
	EnvMonitorDeviceTypeWaterQuality  = 1 // 水质监测仪
	EnvMonitorDeviceTypeWaterVideo    = 2 // 水下视频监控
	EnvMonitorDeviceTypeWorkshopVideo = 3 // 车间视频监控
)

// 环境监测设备管理
type EnvMonitorDevice struct {
	ID                       int64                       `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`                             // Unique ID
	TenantID                 string                      `json:"tenant_id" gorm:"index;comment:租户ID;"`                                        // 租户ID
	DeviceNo                 string                      `json:"device_no" gorm:"size:32;uniqueIndex;comment:环境监测编号;"`                        // 环境监测编号
	Name                     string                      `json:"name" gorm:"size:64;comment:环境监测名称;"`                                         // 环境监测名称
	Type                     int                         `json:"type" gorm:"comment:环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）;"`                       // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	Status                   bool                        `json:"status" gorm:"comment:环境监测设备状态（true=在线，false=离线）;"`                           // 环境监测设备状态（true=在线，false=离线）
	Model                    string                      `json:"model" gorm:"size:64;comment:环境监测设备型号;"`                                      // 环境监测设备型号
	WorkshopID               int64                       `json:"workshop_id" gorm:"index;foreignKey:ID;references:work_shop;comment:关联车间ID;"` // 关联车间ID
	WorkshopName             string                      `json:"workshop_name" gorm:"comment:关联车间名称;"`                                        // 关联车间名称
	CreatedAt                time.Time                   `json:"created_at" gorm:"index;comment:Create time;"`                                // Create time
	UpdatedAt                time.Time                   `json:"updated_at" gorm:"index;comment:Update time;"`                                // Update time
	EnvMonitorDeviceFishPond []*EnvMonitorDeviceFishPond `json:"env_monitor_device_fish_ponds" gorm:"foreignKey:EnvMonitorDeviceID;references:ID;comment:关联鱼池ID;"`
	EnvMonitorDeviceSensor   []*EnvMonitorDeviceSensor   `json:"env_monitor_device_sensors" gorm:"foreignKey:EnvMonitorDeviceID;references:ID;comment:关联传感器ID;"`
}

// Defining the query parameters for the `EnvMonitorDevice` struct.
type EnvMonitorDeviceQueryParam struct {
	util.PaginationParam
	DeviceName string `form:"device_name"`
	Type       *int   `form:"type"`
	WorkshopId string `form:"workshop_id"`
}

// Defining the query options for the `EnvMonitorDevice` struct.
type EnvMonitorDeviceQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDevice` struct.
type EnvMonitorDeviceQueryResult struct {
	Data       EnvMonitorDevices
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDevice` struct.
type EnvMonitorDevices []*EnvMonitorDevice

// Defining the data structure for creating a `EnvMonitorDevice` struct.
type EnvMonitorDeviceForm struct {
	DeviceNo                        string                      `json:"device_no" binding:"required,max=32"` // 环境监测编号
	Name                            string                      `json:"name" binding:"required,max=64"`      // 环境监测名称
	Type                            int                         `json:"type" binding:"required"`             // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	Status                          bool                        `json:"status"`                              // 环境监测设备状态（true=在线，false=离线）
	Model                           string                      `json:"model" binding:"required,max=64"`     // 环境监测设备型号
	WorkshopID                      int64                       `json:"workshop_id" binding:"required"`      // 关联车间ID
	TenantID                        string                      `json:"tenant_id"`                           // 租户ID
	WorkshopName                    string                      `json:"workshop_name" binding:"required"`    // 关联车间名称
	EnvMonitorDeviceDeviceFishPonds []*EnvMonitorDeviceFishPond `json:"env_monitor_device_fish_ponds" gorm:"foreignKey:DeviceID;references:ID;comment:关联鱼池ID;"`
	EnvMonitorDeviceDeviceSensors   []*EnvMonitorDeviceSensor   `json:"env_monitor_device_device_sensors" gorm:"foreignKey:DeviceID;references:ID;comment:关联传感器ID;"`
}

// A validation function for the `EnvMonitorDeviceForm` struct.
func (a *EnvMonitorDeviceForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceForm` to `EnvMonitorDevice` object.
func (a *EnvMonitorDeviceForm) FillTo(envMonitorDevice *EnvMonitorDevice) error {
	envMonitorDevice.DeviceNo = a.DeviceNo
	envMonitorDevice.Name = a.Name
	envMonitorDevice.Type = a.Type
	envMonitorDevice.Status = a.Status
	envMonitorDevice.Model = a.Model
	envMonitorDevice.WorkshopID = a.WorkshopID
	envMonitorDevice.TenantID = a.TenantID
	envMonitorDevice.WorkshopName = a.WorkshopName
	return nil
}

type DeviceStatistics struct {
	Type    int `json:"type"`
	Total   int `json:"total"`
	Online  int `json:"online"`
	Offline int `json:"offline"`
	Rate    int `json:"rate"`
}
