package api

import (
	"ifms/internal/mods/rbac/biz"
	"ifms/internal/mods/rbac/schema"
	"ifms/pkg/util"

	"github.com/gin-gonic/gin"
)

// User management for RBAC
type User struct {
	UserBIZ *biz.User
}

// @Tags UserAPI
// @Security ApiKeyAuth
// @Summary Create user record
// @Param body body schema.UserForm true "Request body"
// @Success 200 {object} util.ResponseResult{data=schema.User}
// @Failure 400 {object} util.ResponseResult
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/users [post]
func (a *User) Create(c *gin.Context) {
	ctx := c.Request.Context()
	item := new(schema.UserForm)
	if err := util.ParseJSON(c, item); err != nil {
		util.ResError(c, err)
		return
	} else if err := item.Validate(); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	item.TenantID = user.TenantID
	_, err := a.UserBIZ.Create(ctx, item)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResOK(c)
}

// @Tags UserAPI
// @Security ApiKeyAuth
// @Summary Query user list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Param username query string false "Username for login"
// @Param name query string false "Name of user"
// @Param status query string false "Status of user (activated, freezed)"
// @Success 200 {object} util.ResponseResult{data=[]schema.User}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/users [get]
func (a *User) Query(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.UserQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}

	result, err := a.UserBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

// @Tags UserAPI
// @Security ApiKeyAuth
// @Summary Query user list
// @Param current query int true "pagination index" default(1)
// @Param pageSize query int true "pagination size" default(10)
// @Param username query string false "Username for login"
// @Param name query string false "Name of user"
// @Param status query string false "Status of user (activated, freezed)"
// @Success 200 {object} util.ResponseResult{data=[]schema.User}
// @Failure 401 {object} util.ResponseResult
// @Failure 500 {object} util.ResponseResult
// @Router /api/v1/queryByRoleId [get]
func (a *User) QueryByRoleId(c *gin.Context) {
	ctx := c.Request.Context()
	var params schema.UserQueryParam
	if err := util.ParseQuery(c, &params); err != nil {
		util.ResError(c, err)
		return
	}
	user := util.FromLoginUser(ctx)
	params.TenantID = user.TenantID

	result, err := a.UserBIZ.Query(ctx, params)
	if err != nil {
		util.ResError(c, err)
		return
	}
	util.ResPage(c, result.Data, result.PageResult)
}

//// @Tags UserAPI
//// @Security ApiKeyAuth
//// @Summary Get user record by ID
//// @Param id path string true "unique id"
//// @Success 200 {object} util.ResponseResult{data=schema.User}
//// @Failure 401 {object} util.ResponseResult
//// @Failure 500 {object} util.ResponseResult
//// @Router /api/v1/users/{id} [get]
//func (a *User) Get(c *gin.Context) {
//	ctx := c.Request.Context()
//	item, err := a.UserBIZ.Get(ctx, c.Param("id"))
//	if err != nil {
//		util.ResError(c, err)
//		return
//	}
//	util.ResSuccess(c, item)
//}
//
//
//// @Tags UserAPI
//// @Security ApiKeyAuth
//// @Summary Update user record by ID
//// @Param id path string true "unique id"
//// @Param body body schema.UserForm true "Request body"
//// @Success 200 {object} util.ResponseResult
//// @Failure 400 {object} util.ResponseResult
//// @Failure 401 {object} util.ResponseResult
//// @Failure 500 {object} util.ResponseResult
//// @Router /api/v1/users/{id} [put]
//func (a *User) Update(c *gin.Context) {
//	ctx := c.Request.Context()
//	item := new(schema.UserForm)
//	if err := util.ParseJSON(c, item); err != nil {
//		util.ResError(c, err)
//		return
//	} else if err := item.Validate(); err != nil {
//		util.ResError(c, err)
//		return
//	}
//
//	err := a.UserBIZ.Update(ctx, c.Param("id"), item)
//	if err != nil {
//		util.ResError(c, err)
//		return
//	}
//	util.ResOK(c)
//}
//
//// @Tags UserAPI
//// @Security ApiKeyAuth
//// @Summary Delete user record by ID
//// @Param id path string true "unique id"
//// @Success 200 {object} util.ResponseResult
//// @Failure 401 {object} util.ResponseResult
//// @Failure 500 {object} util.ResponseResult
//// @Router /api/v1/users/{id} [delete]
//func (a *User) Delete(c *gin.Context) {
//	ctx := c.Request.Context()
//	err := a.UserBIZ.Delete(ctx, c.Param("id"))
//	if err != nil {
//		util.ResError(c, err)
//		return
//	}
//	util.ResOK(c)
//}
//
//// @Tags UserAPI
//// @Security ApiKeyAuth
//// @Summary Reset user password by ID
//// @Param id path string true "unique id"
//// @Success 200 {object} util.ResponseResult
//// @Failure 401 {object} util.ResponseResult
//// @Failure 500 {object} util.ResponseResult
//// @Router /api/v1/users/{id}/reset-pwd [patch]
//func (a *User) ResetPassword(c *gin.Context) {
//	ctx := c.Request.Context()
//	err := a.UserBIZ.ResetPassword(ctx, c.Param("id"))
//	if err != nil {
//		util.ResError(c, err)
//		return
//	}
//	util.ResOK(c)
//}
