package schema

import (
	"time"

	"github.com/go-playground/validator/v10"
	"ifms/internal/config"
	"ifms/pkg/crypto/hash"
	"ifms/pkg/errors"
	"ifms/pkg/util"
)

const (
	UserStatusActivated = 0
	UserStatusFreezed   = 1
)

// User management for RBAC
type User struct {
	ID        int64     `json:"id" gorm:"primarykey;"`         // Unique ID
	Username  string    `json:"username" gorm:"size:64;index"` // Username for login
	Name      string    `json:"name" gorm:"size:64;index"`     // Name of user
	Password  string    `json:"-" gorm:"size:64;"`             // Password for login (encrypted)
	Phone     string    `json:"phone" gorm:"size:32;"`         // Phone number of user
	Email     string    `json:"email" gorm:"size:128;"`        // Email of user
	Remark    string    `json:"remark" gorm:"size:1024;"`      // Remark of user
	Status    int8      `json:"status" gorm:"type:tinyint"`    // Status of user (activated, freezed)
	CreatedAt time.Time `json:"created_at" gorm:"index;"`      // Create time
	UpdatedAt time.Time `json:"updated_at" gorm:"index;"`      // Update time
	Roles     UserRoles `json:"roles" gorm:"-"`                // Roles of user
	TenantID  string    `json:"tenant_id" gorm:"size:32;"`     // Tenant ID
}

func (a *User) TableName() string {
	return config.C.FormatTableName("user")
}

// Defining the query parameters for the `User` struct.
type UserQueryParam struct {
	util.PaginationParam
	LikeUsername string `form:"username"` // Username for login
	LikeName     string `form:"name"`
	RoleId       string `form:"role_id"`                       // role code
	Status       string `form:"status" binding:"oneof=0 1 ''"` // Status of user (activated, freezed)
	TenantID     string `form:"tenant_id"`
}

// Defining the query options for the `User` struct.
type UserQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `User` struct.
type UserQueryResult struct {
	Data       Users
	PageResult *util.PaginationResult
}

// Defining the slice of `User` struct.
type Users []*User

func (a Users) ToIDs() []int64 {
	var ids []int64
	for _, item := range a {
		ids = append(ids, item.ID)
	}
	return ids
}

// Defining the data structure for creating a `User` struct.
type UserForm struct {
	Username string    `json:"username" binding:"required,max=64"` // Username for login
	Name     string    `json:"name" binding:"required,max=64"`     // Name of user
	Password string    `json:"password" binding:"max=64"`          // Password for login (md5 hash)
	Phone    string    `json:"phone" binding:"max=32"`             // Phone number of user
	Email    string    `json:"email" binding:"max=128"`            // Email of user
	Remark   string    `json:"remark" binding:"max=1024"`          // Remark of user
	Roles    UserRoles `json:"roles" binding:"required"`           // Roles of user
	TenantID string    `json:"tenant_id" binding:"max=32"`         // tenant_id
}

// A validation function for the `UserForm` struct.
func (a *UserForm) Validate() error {
	if a.Email != "" && validator.New().Var(a.Email, "email") != nil {
		return errors.BadRequest("", "Invalid email address")
	}
	return nil
}

// Convert `UserForm` to `User` object.
func (a *UserForm) FillTo(user *User) error {
	user.Username = a.Username
	user.Name = a.Name
	user.Phone = a.Phone
	user.Email = a.Email
	user.Remark = a.Remark
	user.TenantID = a.TenantID

	if pass := a.Password; pass != "" {
		hashPass, err := hash.GeneratePassword(pass)
		if err != nil {
			return errors.BadRequest("", "Failed to generate hash password: %s", err.Error())
		}
		user.Password = hashPass
	}

	return nil
}
