package gormx

import (
	"context"
	"fmt"
	"ifms/internal/consts"
	"strings"

	"gorm.io/gorm"
)

// 设置的 key
type tenantKey struct{}

// 设置租户 ID 的上下文函数
func WithTenantID(ctx context.Context, tenantID string) context.Context {
	return context.WithValue(ctx, tenantKey{}, tenantID)
}

func init() {
	//WithTenantID(context.Background(), config.DefaultTenantID)
}

// 获取租户 ID 的上下文函数
func GetTenantID(ctx context.Context) (string, bool) {
	val, ok := ctx.Value(tenantKey{}).(string)
	return val, ok
}

// 插件结构
type Plugin struct {
	ExcludedTables []string // 不注入的表
	ColumnName     string   // 多租户字段名，默认 tenant_id
}

// 注册插件
func (p *Plugin) Name() string {
	return "TenantPlugin"
}

func (p *Plugin) Initialize(db *gorm.DB) error {
	if p.ColumnName == "" {
		p.ColumnName = consts.TenantIDKey
	}

	// 查询前注入 WHERE 条件
	db.Callback().Query().Before("gorm:query").Register("tenant:before_query", p.injectTenantCondition)
	// 创建、更新、删除也可加，但此处只演示查询
	return nil
}

// 注入 tenant 条件
func (p *Plugin) injectTenantCondition(db *gorm.DB) {
	stmt := db.Statement

	// 排除某些表
	if stmt.Schema != nil {
		table := stmt.Schema.Table
		for _, t := range p.ExcludedTables {
			if strings.EqualFold(t, table) {
				return // skip tenant injection
			}
		}
	}

	tenantID, ok := GetTenantID(stmt.Context)
	if !ok || tenantID == "" {
		//TODO : tenant log
		db.Logger.Warn(context.Background(), "TenantPlugin: no tenant ID found in context")
		return
	}

	// 表结构字段校验
	if stmt.Schema != nil {
		if _, exists := stmt.Schema.FieldsByDBName[p.ColumnName]; !exists {
			db.Logger.Warn(context.Background(), fmt.Sprintf("TenantPlugin: column '%s' not found in table '%s'", p.ColumnName, stmt.Table))
			return
		}
	}

	db.Where(fmt.Sprintf("%s = ?", p.ColumnName), tenantID)
}
