package oss

import (
	"context"
	"fmt"
	"ifms/pkg/convert"
	"io"
	"strings"
	"time"

	"github.com/aliyun/aliyun-oss-go-sdk/oss"
)

// AliyunOssClientConfig 阿里云 OSS 客户端配置
type AliyunOssClientConfig struct {
	Endpoint        string
	AccessKeyID     string
	AccessKeySecret string
	BucketName      string
}

// AliyunOssClient 阿里云 OSS 客户端
type AliyunOssClient struct {
	config *AliyunOssClientConfig
	client *oss.Client
}

// NewAliyunOssClient 创建一个新的阿里云 OSS 客户端
func NewAliyunOssClient(config AliyunOssClientConfig) (*AliyunOssClient, error) {
	client, err := oss.New(config.Endpoint, config.AccessKeyID, config.AccessKeySecret)
	if err != nil {
		return nil, err
	}

	return &AliyunOssClient{
		config: &config,
		client: client,
	}, nil
}

// PutObject 上传对象到 OSS
func (c *AliyunOssClient) PutObject(ctx context.Context, bucketName, objectName string, reader io.ReadSeeker, objectSize int64, options ...PutObjectOptions) (*PutObjectResult, error) {
	if bucketName == "" {
		bucketName = c.config.BucketName
	}

	//objectName = formatObjectName(c.config.Prefix, objectName)

	var opt PutObjectOptions
	if len(options) > 0 {
		opt = options[0]
	}

	bucket, err := c.client.Bucket(bucketName)
	if err != nil {
		return nil, err
	}

	// 设置对象为公共读取权限
	err = bucket.PutObject(objectName, reader,
		oss.ObjectACL(oss.ACLPublicRead),
		oss.Meta("Content-Type", opt.ContentType),
	)
	if err != nil {
		return nil, err
	}

	// 生成公共访问 URL
	url := fmt.Sprintf("https://%s.%s/%s", bucketName, c.config.Endpoint, objectName)

	return &PutObjectResult{
		URL:  url,
		Key:  objectName,
		Size: objectSize,
	}, nil
}

// GetObject 从 OSS 获取对象
func (c *AliyunOssClient) GetObject(ctx context.Context, bucketName, objectName string) (io.ReadCloser, error) {
	if bucketName == "" {
		bucketName = c.config.BucketName
	}

	//objectName = formatObjectName(c.config.Prefix, objectName)

	bucket, err := c.client.Bucket(bucketName)
	if err != nil {
		return nil, err
	}

	return bucket.GetObject(objectName)
}

// RemoveObject 删除 OSS 中的对象
func (c *AliyunOssClient) RemoveObject(ctx context.Context, bucketName, objectName string) error {
	if bucketName == "" {
		bucketName = c.config.BucketName
	}

	//objectName = formatObjectName(c.config.Prefix, objectName)

	bucket, err := c.client.Bucket(bucketName)
	if err != nil {
		return err
	}

	return bucket.DeleteObject(objectName)
}

// RemoveObjectByURL 根据 URL 删除对象
func (c *AliyunOssClient) RemoveObjectByURL(ctx context.Context, urlStr string) error {
	prefix := c.config.BucketName + "."
	if !strings.HasPrefix(urlStr, prefix) {
		return nil
	}

	objectName := strings.TrimPrefix(urlStr, prefix)
	return c.RemoveObject(ctx, c.config.BucketName, objectName)
}

// StatObject 获取对象的状态信息
func (c *AliyunOssClient) StatObject(ctx context.Context, bucketName, objectName string) (*ObjectStat, error) {
	if bucketName == "" {
		bucketName = c.config.BucketName
	}

	//objectName = formatObjectName(c.config.Prefix, objectName)

	bucket, err := c.client.Bucket(bucketName)
	if err != nil {
		return nil, err
	}

	// 获取对象的元数据
	metadata, err := bucket.GetObjectDetailedMeta(objectName)
	if err != nil {
		return nil, err
	}

	size, _ := metadata["Content-Length"]
	//lastModified, _ := metadata["Last-Modified"]
	contentType, _ := metadata["Content-Type"]
	etag, _ := metadata["ETag"]

	// 提取自定义元数据
	var userMetadata map[string]string
	if metadata != nil {
		userMetadata = make(map[string]string)
		for key, value := range metadata {
			if strings.HasPrefix(key, oss.HTTPHeaderOssMetaPrefix) {
				// 去掉前缀 x-oss-meta-
				userKey := strings.TrimPrefix(key, oss.HTTPHeaderOssMetaPrefix)
				userMetadata[userKey] = value[0]
			}
		}
	}

	return &ObjectStat{
		Key:          objectName,
		ETag:         etag[0],
		LastModified: time.Time{}, // 需要解析 Last-Modified 时间
		Size:         convert.ToInt64(size[0]),
		ContentType:  contentType[0],
		UserMetadata: userMetadata,
	}, nil
}

// StatObjectByURL 根据 URL 获取对象状态
func (c *AliyunOssClient) StatObjectByURL(ctx context.Context, urlStr string) (*ObjectStat, error) {
	prefix := c.config.BucketName + "."
	if !strings.HasPrefix(urlStr, prefix) {
		return nil, nil
	}

	objectName := strings.TrimPrefix(urlStr, prefix)
	return c.StatObject(ctx, c.config.BucketName, objectName)
}
